/*
 *  Copyright (C) 2014 Benjamin W. (bitbatzen@gmail.com)
 *
 *  This file is part of Netvisix.
 *
 *  Netvisix is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Netvisix is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Netvisix.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "StatisticPopup.h"
#include "ui_StatisticPopup.h"
#include "Net/NetStatistic.h"
#include "Net/NetEvent.h"
#include "Net/NetEventManager.h"

#include <QDateTime>
#include <QTimer>
#include <QTableWidget>

#include <iomanip>


namespace Netvisix {

    StatisticPopup::StatisticPopup(NetStatistic* statistic, std::string title, QWidget* parent) :
            QWidget(parent, Qt::Window),
            ui(new Ui::StatisticPopup) {

        ui->setupUi(this);

        setAttribute(Qt::WA_DeleteOnClose);

        this->statistic = statistic;

        QPalette Pal(palette());
        Pal.setColor(QPalette::Background, QColor(200, 200, 200, 255));
        setAutoFillBackground(true);
        setPalette(Pal);

        lastUpdateTime = 0;

        setWindowTitle(title.c_str());

        timer = new QTimer();
        QObject::connect(timer, SIGNAL(timeout()), this, SLOT(updateLoop()));
        timer->start();

        items = new std::vector<StatisticItem*>();

        float frameWidth = START_POS_X + 5 * OFFSET_X;
        float frameHeight = START_POS_Y + 26 * OFFSET_Y + 10;
        setFixedSize(frameWidth, frameHeight);

        // gui layout
        const float columnTitlePosY = 10;
        createColumnTitleLabel(START_POS_X + 1 * OFFSET_X, columnTitlePosY, "Pckts from");
        createColumnTitleLabel(START_POS_X + 2 * OFFSET_X, columnTitlePosY, "Pckts to");
        createColumnTitleLabel(START_POS_X + 3 * OFFSET_X, columnTitlePosY, "Bytes from");
        createColumnTitleLabel(START_POS_X + 4 * OFFSET_X, columnTitlePosY, "Bytes to");

        addItem(statistic->getItem(IPVersion::ALL, Protocol::ETHERNETII), "Frames", 0);
        addItem(statistic->getItem(IPVersion::ALL, Protocol::UNKOWN), "Unkown", 1);

        // IPv4
        createSubTitleLabel(START_POS_X, START_POS_Y + 2.9f * OFFSET_Y, "IPv4");
        createLine(START_POS_Y + 3.55f * OFFSET_Y);

        addItem(statistic->getItem(IPVersion::IPV4, Protocol::ARP), "ARP", 4);

        addItem(statistic->getItem(IPVersion::IPV4, Protocol::ICMP), "ICMP", 6);
        addItem(statistic->getItem(IPVersion::IPV4, Protocol::IGMP), "IGMP", 7);

        addItem(statistic->getItem(IPVersion::IPV4, Protocol::TCP), "TCP", 9);
        addItem(statistic->getItem(IPVersion::IPV4, Protocol::UDP), "UDP", 10);

        addItem(statistic->getItem(IPVersion::IPV4, Protocol::DNS), "DNS", 12);
        addItem(statistic->getItem(IPVersion::IPV4, Protocol::DHCP), "DHCP", 13);
        addItem(statistic->getItem(IPVersion::IPV4, Protocol::HTTP), "HTTP", 14);

        // IPv6
        createSubTitleLabel(START_POS_X, START_POS_Y + 15.9f * OFFSET_Y, "IPv6");
        createLine(START_POS_Y + 16.55f * OFFSET_Y);

        addItem(statistic->getItem(IPVersion::IPV6, Protocol::ICMPv6), "ICMPv6", 17);

        addItem(statistic->getItem(IPVersion::IPV6, Protocol::TCP), "TCP", 19);
        addItem(statistic->getItem(IPVersion::IPV6, Protocol::UDP), "UDP", 20);

        addItem(statistic->getItem(IPVersion::IPV6, Protocol::DNS), "DNS", 22);
        addItem(statistic->getItem(IPVersion::IPV6, Protocol::DHCPv6), "DHCPv6", 23);
        addItem(statistic->getItem(IPVersion::IPV6, Protocol::HTTP), "HTTP", 24);


        ui->labelInfo->move(frameWidth / 2 - ui->labelInfo->width() / 2, frameHeight - 25);

        updateStatisticDisplay();
    }

    StatisticPopup::~StatisticPopup() {
        delete timer;
        delete ui;

        for (unsigned int i = 0; i < items->size(); i++) {
            delete items->at(i);
        }
        delete items;
    }

    void StatisticPopup::addItem(NetStatistic::Item *item, std::string name, int y) {
        float posY = START_POS_Y + y * OFFSET_Y;

        QFont fontName = QFont();
        fontName.setFamily("SansSerif");
        fontName.setPointSize(11);
        fontName.setBold(true);
        StatisticItem* sItem = new StatisticItem();

        sItem->item = item;

        sItem->labelName = new QLabel(name.c_str(), this);
        sItem->labelName->move(START_POS_X, posY);
        sItem->labelName->setStyleSheet("QLabel { color: rgb(70, 70, 130) }");
        sItem->labelName->setFont(fontName);

        sItem->labelPacketsSnt = createCounterLabel(START_POS_X + 1 * OFFSET_X, posY);
        sItem->labelPacketsRcv = createCounterLabel(START_POS_X + 2 * OFFSET_X, posY);
        sItem->labelBytesSnt = createCounterLabel(START_POS_X + 3 * OFFSET_X, posY);
        sItem->labelBytesRcv = createCounterLabel(START_POS_X + 4 * OFFSET_X, posY);

        items->push_back(sItem);
    }

    void StatisticPopup::createColumnTitleLabel(float posX, float posY, std::string text) {
        QFont font = QFont();
        font.setFamily("SansSerif");
        font.setPointSize(11);
        font.setBold(true);

        QLabel* label = new QLabel(text.c_str(), this);
        label->move(posX, posY);
        label->setFont(font);
        label->setFixedWidth(200);
        label->setStyleSheet("QLabel { color: rgb(70, 70, 130) }");
    }

    void StatisticPopup::createSubTitleLabel(float posX, float posY, std::string text) {
        QFont font = QFont();
        font.setFamily("SansSerif");
        font.setPointSize(11);
        font.setBold(true);
        font.setItalic(true);

        QLabel* label = new QLabel(text.c_str(), this);
        label->move(posX, posY);
        label->setFont(font);
        label->setFixedWidth(200);
        label->setStyleSheet("QLabel { color: rgb(130, 130, 180) }");
    }

    void StatisticPopup::createLine(float posY) {
        QFrame* line = new QFrame(this);
        line->setGeometry(QRect(8, posY, width() - 16, 2));

        QPalette Pal(palette());
        Pal.setColor(QPalette::Background, QColor(160, 160, 180));
        line->setAutoFillBackground(true);
        line->setPalette(Pal);
    }

    QLabel* StatisticPopup::createCounterLabel(float posX, float posY) {
        QFont font = QFont();
        font.setFamily("SansSerif");
        font.setPointSize(11);
        font.setBold(true);

        QLabel* label = new QLabel(this);
        label->move(posX, posY);
        label->setFont(font);
        label->setFixedWidth(200);

        return label;
    }

    void StatisticPopup::updateStatisticDisplay() {
        for (unsigned int i = 0; i < items->size(); i++) {
            StatisticItem* sItem = items->at(i);

            handleCountTextColor(sItem->labelPacketsSnt, sItem->item->packetsSnt);
            sItem->labelPacketsSnt->setText(QString::number(sItem->item->packetsSnt));

            handleCountTextColor(sItem->labelPacketsRcv, sItem->item->packetsRcv);
            sItem->labelPacketsRcv->setText(QString::number(sItem->item->packetsRcv));

            handleCountTextColor(sItem->labelBytesSnt, sItem->item->bytesSnt);
            sItem->labelBytesSnt->setText(getByteString(sItem->item->bytesSnt).c_str());

            handleCountTextColor(sItem->labelBytesRcv, sItem->item->bytesRcv);
            sItem->labelBytesRcv->setText(getByteString(sItem->item->bytesRcv).c_str());
        }
    }

    void StatisticPopup::handleCountTextColor(QLabel* label, int value) {
        if (value == 0) {
            label->setStyleSheet("QLabel { color: rgb(180, 180, 180) }");
        }
        else {
            label->setStyleSheet("QLabel { color: rgb(0, 0, 0) }");
        }
    }

    std::string StatisticPopup::getByteString(unsigned long long byteCount) {
        std::string byteString = "";

        if (byteCount < 1024.0) {
            byteString = std::to_string(byteCount);
            byteString += " B";
            return byteString;
        }

        std::stringstream ss;

        double byteCountD = (double) byteCount / 1024.0;
        if (byteCountD < 1024.0) {
            ss << std::fixed << std::setprecision(2) << byteCountD;
            byteString = ss.str();
            byteString += " KiB";
            return byteString;
        }

        byteCountD /= 1024.0;
        if (byteCountD < 1024.0) {
            ss << std::fixed << std::setprecision(2) << byteCountD;
            byteString = ss.str();
            byteString += " MiB";
            return byteString;
        }

        byteCountD /= 1024.0;
        ss << std::fixed << std::setprecision(2) << byteCountD;
        byteString = ss.str();
        byteString += " GiB";
        return byteString;
    }

    void StatisticPopup::updateLoop() {
        if (NetEventManager::SharedInstance()->getIsSniffingRunning() == false) {
            close();
            return;
        }

        if (QDateTime::currentMSecsSinceEpoch() - lastUpdateTime >= 300) {
            updateStatisticDisplay();
            lastUpdateTime = QDateTime::currentMSecsSinceEpoch();
        }
    }

} // namespace Netvisix
